#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import os
import sys

logging.basicConfig(level=logging.INFO)
sys.path.append(os.path.abspath('.'))

from nico.consumer import Consumer
from nico.utils import load_modules
from nico.load_settings import load_settings


def main():
    settings = load_settings()
    topic = settings['topic']
    consumer_group = settings['consumer_group']
    jobspath = settings['jobspath']
    load_modules(jobspath)
    consumer = Consumer(topic, consumer_group, settings)
    consumer.run()


if __name__ == '__main__':
    """ python examples/main.py -t default -g test -c examples/config.json """
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os


def load_modules(path):
    dir_ = path.replace('.', '/')
    paths = []
    for filename in os.listdir(dir_):
        if filename.endswith('.py'):
            paths.append(path + '.' + filename[:-3])

    for path in paths:
        __import__(path)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from nico.register import on
from nico.consts import DEFAULT_TOPIC


@on('MESSAGE1', DEFAULT_TOPIC)
def test(data):
    print('test', data)


@on('MESSAGE1', DEFAULT_TOPIC)
def test2(data):
    print('test2', data)


@on('MESSAGE1', DEFAULT_TOPIC)
def test3(data):
    return 1 / 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from functools import wraps

from nico.consts import DEFAULT_TOPIC


class Register(object):

    registers = {}

    @classmethod
    def generate_key(cls, topic, message_type):
        return '{}:{}'.format(topic, message_type)

    def register(self, topic, message_type, func):
        key = self.generate_key(topic, message_type)
        if key not in self.registers:
            self.registers[key] = []
        self.registers[key].append(func)

    def get_jobs(self, topic, message_type):
        key = self.generate_key(topic, message_type)
        return self.registers[key] if key in self.registers else []


register = Register()


def on(message_type, topic=DEFAULT_TOPIC):
    def decorate(func):
        register.register(topic, message_type, func)

        @wraps(func)
        def wrapper(*args, **kwargs):
            return func(*args, **kwargs)
        return wrapper
    return decorate



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Nico

Usage:
    test_job1.py -c file -t topic -g group
    test_job1.py -h

Options:
    -h --help       show this
    -v --version    show version
    -t topic        kafka topic
    -g group        kafka consumer group
    -c file         config file name

config file:
    {
        "jobspath": "",
        "kafka_hosts": ""
    }
"""

import json

from docopt import docopt


def load_settings():
    arguments = docopt(__doc__, version='Nico 0.1')
    filename = arguments['-c']
    json_config = json.load(file(filename))
    settings = {
        'topic': arguments['-t'],
        'consumer_group': arguments['-g'],
        'kafka_hosts': json_config['kafka_hosts'],
        'jobspath': json_config['jobspath']
    }
    return settings



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import logging
import json

from nico.register import register
from nico.kafka_client import get_kafka_client


def message_parser(message):
    try:
        message = json.loads(message)
        message_type = message['type']
        data = message['data']
        created = message['created']
        return message_type, data, created
    except Exception as e:
        logging.error('message parse error', exc_info=True)
        return None


class Consumer(object):

    def __init__(self, topic, consumer_group, settings):
        self.topic = topic
        self.consumer_group = consumer_group
        self.settings = settings
        self.client = get_kafka_client(self.settings['kafka_hosts'])

    def get_consumer(self):
        # from examples.messages import test_message1, test_message2
        # kafka_consumer = [
        #     json.dumps(test_message1()),
        #     json.dumps(test_message2()),
        # ]
        t = self.client.topics[self.topic]
        kafka_consumer = t.get_balanced_consumer(consumer_group=self.consumer_group, managed=True)
        return kafka_consumer

    def run(self):
        consumer = self.get_consumer()
        for message in consumer:
            parser = message_parser(message)
            if parser:
                message_type, data, created = message_parser(message)
                jobs = register.get_jobs(self.topic, message_type)
                for job in jobs:
                    try:
                        job(data)
                        logging.info('job {} run success, topic={}, message_type={}'
                                     .format(job.__name__, self.topic, message_type))
                    except Exception as e:
                        # TODO: retry 三次
                        logging.error('job {} run error'.format(job.__name__), exc_info=True)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

setup(
    name='nico',
    version='0.1',
    packages=find_packages(),
    install_requires=[
        'docopt',
        'pykafka>=2.7.0'
    ],
    author='omsobliga',
    author_email='omsobliga@gmail.com',
    description='job scheduler for kafka message',
    license='MIT',
    url='https://github.com/omsobliga',
    entry_points={
        'console_scripts': [
            'nico = nico.main:main'
        ],
    }
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time


def test_message1(a='a', b=1):
    return {
        'type': 'MESSAGE1',
        'data': {
            'a': a,
            'b': b
        },
        'created': int(time.time())
    }


def test_message2(a='a', b=1):
    return {
        'type': 'MESSAGE2',
        'data': {
            'a': a,
            'b': b
        },
        'created': int(time.time())
    }
